@@ -15,7 +15,7 @@ import uuid
 import numpy as np
 from tqdm import tqdm
 
-from cloudfiles import CloudFiles
+from cloudfiles import CloudFiles, compression
 
 from cloudvolume import lib, exceptions
 from ....lib import Bbox, Vec, sip, first
@@ -95,7 +95,7 @@ class PrecomputedImageSource(ImageSourceInterface):
     mip = mip if mip is not None else self.config.mip
     mip = self.meta.to_mip(mip)
     
-    cf = CloudFiles(self.meta.cloudpath)
+    cf = CloudFiles(self.meta.cloudpath, secrets=self.config.secrets)
     key = self.meta.key(mip)
     return len(list(sip(cf.list(prefix=key), 1))) > 0
 
@@ -161,6 +161,7 @@ class PrecomputedImageSource(ImageSourceInterface):
         compress=self.config.compress,
         order=order,
         green=self.config.green,
+        secrets=self.config.secrets,
       )
 
   @readonlyguard
@@ -194,7 +195,7 @@ class PrecomputedImageSource(ImageSourceInterface):
     if self.is_sharded(mip):
       (filename, shard) = self.make_shard(image, bbox, mip)
       basepath = self.meta.join(self.meta.cloudpath, self.meta.key(mip))
-      CloudFiles(basepath, progress=self.config.progress).put(
+      CloudFiles(basepath, progress=self.config.progress, secrets=self.config.secrets).put(
         filename, shard, 
         compress=self.config.compress, 
         cache_control=self.config.cdn_cache
@@ -237,7 +238,10 @@ class PrecomputedImageSource(ImageSourceInterface):
       protocol=self.meta.path.protocol
     )
 
-    return CloudFiles(self.meta.cloudpath, progress=self.config.progress).exists(cloudpaths)
+    return CloudFiles(
+      self.meta.cloudpath, progress=self.config.progress,
+      secrets=self.config.secrets
+    ).exists(cloudpaths)
 
   @readonlyguard
   def delete(self, bbox, mip=None):
@@ -269,11 +273,11 @@ class PrecomputedImageSource(ImageSourceInterface):
       protocol=self.meta.path.protocol
     ) # need to regenerate so that generator isn't used up
 
-    CloudFiles(self.meta.cloudpath, progress=self.config.progress) \
+    CloudFiles(self.meta.cloudpath, progress=self.config.progress, secrets=self.config.secrets) \
       .delete(cloudpaths())
 
     if self.cache.enabled:
-      CloudFiles('file://' + self.cache.path, progress=self.config.progress) \
+      CloudFiles('file://' + self.cache.path, progress=self.config.progress, secrets=self.config.secrets) \
         .delete(cloudpaths())
 
   def transfer_to(self, cloudpath, bbox, mip, block_size=None, compress=True, compress_level=None):
@@ -353,23 +357,31 @@ class PrecomputedImageSource(ImageSourceInterface):
       total=num_blocks,
     )
 
-    cfsrc = CloudFiles(self.meta.cloudpath)
+    cfsrc = CloudFiles(self.meta.cloudpath, secrets=self.config.secrets)
     cfdest = CloudFiles(cloudpath)
 
+    def check(files):
+      errors = [
+        file for file in files if \
+        (file['content'] is None or file['error'] is not None)
+      ]
+      if errors:
+        error_paths = [ f['path'] for f in errors ]
+        raise exceptions.EmptyFileException("{} were empty or had IO errors.".format(", ".join(error_paths)))
+      return files
+
     with pbar:
-      for _ in range(num_blocks, 0, -1):
-        srcpaths = list(itertools.islice(cloudpaths, step))
-        files = cfsrc.get(srcpaths)
-        files = [ (f['path'], f['content']) for f in files ]
+      for srcpaths in sip(cloudpaths, step):
+        files = check(cfsrc.get(srcpaths, raw=True))
         cfdest.puts(
-          files, 
-          compress=compress, 
-          compression_level=compress_level,
+          compression.transcode(files, encoding=compress, level=compress_level), 
+          compress=compress,
           content_type=tx.content_type(destvol),
+          raw=True
         )
         pbar.update()
 
-  def make_shard(self, img, bbox, mip=None, spec=None):
+  def make_shard(self, img, bbox, mip=None, spec=None, progress=False):
     """
     Convert an image that represents a single complete shard 
     into a shard file.
@@ -392,30 +404,37 @@ class PrecomputedImageSource(ImageSourceInterface):
       else:
         raise ValueError("mip {} does not have a sharding specification.".format(mip))
 
+    bbox = Bbox.create(bbox)
     if bbox.subvoxel():
       raise ValueError("Bounding box is too small to make a shard. Got: {}".format(bbox))
 
-    bbox = Bbox.create(bbox, self.meta.bounds(mip), bounded=True)
+    # Alignment Checks:
+    # 1. Aligned to atomic chunks - required for grid point generation
     aligned_bbox = bbox.expand_to_chunk_size(
       self.meta.chunk_size(mip), offset=self.meta.voxel_offset(mip)
     )
-    aligned_bbox = Bbox.clamp(aligned_bbox, self.meta.bounds(mip))
-
     if bbox != aligned_bbox:
       raise exceptions.AlignmentError(
         "Unable to create shard from a non-chunk aligned bounding box. Requested: {}, Aligned: {}".format(
         bbox, aligned_bbox
       ))
 
+    # 2. Covers the dataset at least partially
+    aligned_bbox = Bbox.clamp(aligned_bbox, self.meta.bounds(mip))
+    if aligned_bbox.subvoxel():
+      raise exceptions.OutOfBoundsError("Shard completely outside dataset: Requested: {}, Dataset: {}".format(
+        bbox, self.meta.bounds(mip)
+      ))
+
     grid_size = self.grid_size(mip)
     chunk_size = self.meta.chunk_size(mip)
     reader = sharding.ShardReader(self.meta, self.cache, spec)
 
-    gpts = lambda: gridpoints(bbox, self.meta.bounds(mip), chunk_size)
+    # 3. Gridpoints all within this one shard
+    gpts = list(gridpoints(aligned_bbox, self.meta.bounds(mip), chunk_size))
+    morton_codes = compressed_morton_code(gpts, grid_size)
     all_same_shard = bool(reduce(lambda a,b: operator.eq(a,b) and a,
-      map(reader.get_filename,
-        map(lambda gpt: compressed_morton_code(gpt, grid_size), gpts())
-      )
+      map(reader.get_filename, morton_codes)
     ))
 
     if not all_same_shard:
@@ -424,17 +443,20 @@ class PrecomputedImageSource(ImageSourceInterface):
       )
 
     labels = {}
-    pt_anchor = next(gpts())
-    for pt_abs in gpts():
-      pt_rel = pt_abs - pt_anchor
-      cutout_bbx = Bbox(pt_rel * chunk_size, (pt_rel + 1) * chunk_size)
+    pt_anchor = gpts[0] * chunk_size
+    for pt_abs, morton_code in zip(gpts, morton_codes):
+      cutout_bbx = Bbox(pt_abs * chunk_size, (pt_abs + 1) * chunk_size)
+
+      # Neuroglancer expects border chunks not to extend beyond dataset bounds
+      cutout_bbx.maxpt = cutout_bbx.maxpt.clip(None, self.meta.volume_size(mip))
+      cutout_bbx -= pt_anchor
+
       chunk = img[ cutout_bbx.to_slices() ]
-      morton_code = compressed_morton_code(pt_abs, grid_size)
       labels[morton_code] = chunks.encode(chunk, self.meta.encoding(mip))
 
     shard_filename = reader.get_filename(first(labels.keys()))
 
-    return (shard_filename, spec.synthesize_shard(labels))
+    return (shard_filename, spec.synthesize_shard(labels, progress=progress))
 
   def is_sharded(self, mip):
     scale = self.meta.scale(mip)
