@@ -7,7 +7,7 @@ import numpy as np
 from six.moves import range
 from tqdm import tqdm
 
-from cloudfiles import reset_connection_pools, CloudFiles
+from cloudfiles import reset_connection_pools, CloudFiles, compression
 
 from ....exceptions import EmptyVolumeException, EmptyFileException
 from ....lib import (  
@@ -55,16 +55,16 @@ def download_sharded(
 
   renderbuffer = np.zeros(shape=shape, dtype=meta.dtype, order=order)
 
-  gpts = gridpoints(full_bbox, bounds, chunk_size)
+  gpts = list(gridpoints(full_bbox, bounds, chunk_size))
 
   code_map = {}
-  for gridpoint in gpts:
-    zcurve_code = compressed_morton_code(gridpoint, grid_size)
+  morton_codes = compressed_morton_code(gpts, grid_size)
+  for gridpoint, morton_code in zip(gpts, morton_codes):
     cutout_bbox = Bbox(
       bounds.minpt + gridpoint * chunk_size,
       min2(bounds.minpt + (gridpoint + 1) * chunk_size, bounds.maxpt)
     )
-    code_map[zcurve_code] = cutout_bbox
+    code_map[morton_code] = cutout_bbox
 
   all_chunkdata = reader.get_data(list(code_map.keys()), meta.key(mip), progress=progress)
   for zcode, chunkdata in all_chunkdata.items():
@@ -94,7 +94,7 @@ def download(
     parallel, location, 
     retain, use_shared_memory, 
     use_file, compress, order='F',
-    green=False
+    green=False, secrets=None
   ):
   """Cutout a requested bounding box from storage and return it as a numpy array."""
   
@@ -142,7 +142,7 @@ def download(
       meta, cache, mip, cloudpaths, 
       fn=process, fill_missing=fill_missing,
       progress=progress, compress_cache=compress_cache, 
-      green=green 
+      green=green, secrets=secrets 
     )
   else:
     handle, renderbuffer = multiprocess_download(
@@ -153,6 +153,7 @@ def download(
       use_shared_memory=(use_file == False),
       order=order,
       green=green,
+      secrets=secrets,
     )
   
   return VolumeCutout.from_volume(
@@ -166,7 +167,7 @@ def multiprocess_download(
     fill_missing, progress,
     parallel, location, 
     retain, use_shared_memory, order,
-    green
+    green, secrets=None
   ):
 
   cloudpaths_by_process = []
@@ -181,7 +182,7 @@ def multiprocess_download(
     requested_bbox, 
     fill_missing, progress,
     location, use_shared_memory,
-    green
+    green, secrets
   )
   parallel_execution(cpd, cloudpaths_by_process, parallel, cleanup_shm=location)
 
@@ -212,7 +213,7 @@ def child_process_download(
     dest_bbox, 
     fill_missing, progress,
     location, use_shared_memory, green,
-    cloudpaths
+    secrets, cloudpaths
   ):
   reset_connection_pools() # otherwise multi-process hangs
 
@@ -237,7 +238,7 @@ def child_process_download(
     meta, cache, mip, cloudpaths,
     fn=process, fill_missing=fill_missing,
     progress=progress, compress_cache=compress_cache,
-    green=green
+    green=green, secrets=secrets
   )
 
   array_like.close()
@@ -246,17 +247,25 @@ def download_chunk(
     meta, cache, 
     cloudpath, mip,
     filename, fill_missing,
-    enable_cache, compress_cache
+    enable_cache, compress_cache,
+    secrets
   ):
-  content = CloudFiles(cloudpath).get(filename)
+  (file,) = CloudFiles(cloudpath, secrets=secrets).get([ filename ], raw=True)
+  content = file['content']
 
   if enable_cache:
+    cache_content = next(compression.transcode(file, compress_cache))['content'] 
     CloudFiles('file://' + cache.path).put(
       path=filename, 
-      content=(content or b''), 
+      content=(cache_content or b''), 
       content_type=content_type(meta.encoding(mip)), 
       compress=compress_cache,
+      raw=bool(cache_content),
     )
+    del cache_content
+
+  if content is not None:
+    content = compression.decompress(content, file['compress'])
 
   bbox = Bbox.from_filename(filename) # possible off by one error w/ exclusive bounds
   img3d = decode(meta, filename, content, fill_missing, mip)
@@ -265,7 +274,7 @@ def download_chunk(
 def download_chunks_threaded(
     meta, cache, mip, cloudpaths, fn, 
     fill_missing, progress, compress_cache,
-    green=False
+    green=False, secrets=None
   ):
   locations = cache.compute_data_locations(cloudpaths)
   cachedir = 'file://' + os.path.join(cache.path, meta.key(mip))
@@ -274,7 +283,8 @@ def download_chunks_threaded(
     img3d, bbox = download_chunk(
       meta, cache, cloudpath, mip,
       filename, fill_missing,
-      enable_cache, compress_cache
+      enable_cache, compress_cache,
+      secrets
     )
     fn(img3d, bbox)
 
@@ -287,10 +297,13 @@ def download_chunks_threaded(
 
   downloads = itertools.chain( local_downloads, remote_downloads )
 
+  if progress and not isinstance(progress, str):
+    progress = "Downloading"
+
   schedule_jobs(
     fns=downloads, 
     concurrency=DEFAULT_THREADS, 
-    progress=('Downloading' if progress else None),
+    progress=progress,
     total=len(cloudpaths),
     green=green,
   )
@@ -311,7 +324,7 @@ def decode(meta, input_bbox, content, fill_missing, mip):
 
   if not content:
     if fill_missing:
-      content = ''
+      content = b''
     else:
       raise EmptyVolumeException(input_bbox)
 
